@@ -83,7 +83,7 @@
 #define GCONF_OSSO_SOUND_DIR "/apps/osso/sound"
 #define GCONF_MASTER_VOLUME GCONF_OSSO_SOUND_DIR "/master_volume"
 
-//#define VOLUME_BEEP_DELAY 300
+#define BRIGHTNESS_CHANGE_DELAY 300
 
 
 #define GCONF_PREFIX "/apps/adv-backlight"
@@ -142,7 +142,7 @@ struct _AdvancedBacklightPluginPrivate
   gint is_770;
 
   gint notify_id;
-  //guint timer_id;
+  guint timer_id;
   struct timeval last_beep;
   gint brightness_jump;
   gint volume_jump;
@@ -195,8 +195,10 @@ static gpointer advanced_backlight_plugin_dsme_update_brightness_thread (gpointe
 /*static void advanced_backlight_plugin_dsme_update_brightness (AdvancedBacklightPlugin *);*/
 static void advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp);
 static void advanced_backlight_plugin_load_icons (AdvancedBacklightPlugin * vsp);
-static gboolean volume_beep (GtkWidget *w, GdkEventButton *e, AdvancedBacklightPlugin * vsp);
+static gboolean volume_beep (GtkWidget * w, GdkEventButton * e, AdvancedBacklightPlugin * vsp);
 
+static gboolean brightness_level_changed (AdvancedBacklightPlugin * vsp);
+static void brightness_level_changing (GtkWidget * widget, AdvancedBacklightPlugin * vsp);
 
 static void advanced_backlight_plugin_check_init_rotation (AdvancedBacklightPlugin * vsp);
 static gint advanced_backlight_plugin_get_rotation (AdvancedBacklightPlugin * vsp);
@@ -293,9 +295,9 @@ advanced_backlight_plugin_init (AdvancedBacklightPlugin * vsp)
   priv->is_770 = is_770 ();
 
   if (priv->is_770)
-	  priv->max_brightness = 126;
+    priv->max_brightness = 126;
   else
-	  priv->max_brightness = 127;
+    priv->max_brightness = 127;
 
   gettimeofday (&priv->last_beep, NULL);
   advanced_backlight_plugin_load_settings (vsp);
@@ -742,27 +744,44 @@ rotation_button_clicked (GtkButton * button, AdvancedBacklightPlugin * vsp)
 
 }
 
-static void
-brightness_level_changed (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
+static gboolean
+brightness_level_changed (AdvancedBacklightPlugin * vsp)
 {
+  GDK_THREADS_ENTER();
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
   int *value;
 
   g_debug ("%s\n", __PRETTY_FUNCTION__);
-
-  priv->brightness_level = (gint) hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider));
-
-  g_string_printf (priv->brightness_level_string, "%3d", priv->brightness_level);
-  gtk_label_set_text (GTK_LABEL (priv->brightness_level_label), (priv->brightness_level_string)->str);
-  g_debug ("priv->brightness_level=%d\n", priv->brightness_level);
   advanced_backlight_plugin_update_icons (vsp);
 
   value = g_new (int, 1);
   *value = priv->brightness_level;
+  g_debug ("priv->brightness_level=%d\n", priv->brightness_level);
   g_async_queue_push (priv->brightness_queue, value);
+  GDK_THREADS_LEAVE();
+  return FALSE;
 }
+
+static void
+brightness_level_changing (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
+{
+  AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
+
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
+  priv->brightness_level = (gint) hildon_volumebar_get_level (HILDON_VOLUMEBAR (priv->brightness_slider));
+
+  g_string_printf (priv->brightness_level_string, "%3d", priv->brightness_level);
+  gtk_label_set_text (GTK_LABEL (priv->brightness_level_label), (priv->brightness_level_string)->str);
+
+  if (priv->timer_id > 0)
+  {
+    g_source_remove(priv->timer_id);
+  }
+  priv->timer_id = g_timeout_add (BRIGHTNESS_CHANGE_DELAY, (GSourceFunc) brightness_level_changed, vsp);
+}
+
 static gboolean
-volume_beep (GtkWidget *w, GdkEventButton *e, AdvancedBacklightPlugin * vsp)
+volume_beep (GtkWidget * w, GdkEventButton * e, AdvancedBacklightPlugin * vsp)
 {
   AdvancedBacklightPluginPrivate *priv = ADVANCED_BACKLIGHT_PLUGIN_GET_PRIVATE (vsp);
   g_debug ("%s\n", __PRETTY_FUNCTION__);
@@ -779,6 +798,7 @@ advanced_backlight_plugin_update_volume_value (AdvancedBacklightPlugin * vsp, gi
   gconf_client_set_int (priv->gconf_client, GCONF_MASTER_VOLUME, volume, NULL);
   g_debug ("%s ->\n", __PRETTY_FUNCTION__);
 }
+
 /*
 static gpointer
 advanced_backlight_plugin_update_volume_thread (gpointer d)
@@ -840,7 +860,7 @@ advanced_backlight_plugin_update_volume (AdvancedBacklightPlugin * vsp)
   //value = g_new (int, 1);
   //*value = volume;
   //g_async_queue_push (priv->volume_queue, value);
-  advanced_backlight_plugin_update_volume_value(vsp, volume);
+  advanced_backlight_plugin_update_volume_value (vsp, volume);
 }
 
 static void
@@ -851,12 +871,12 @@ volume_level_changed (GtkWidget * widget, AdvancedBacklightPlugin * vsp)
   g_debug ("%s\n", __PRETTY_FUNCTION__);
   /* We only want to beep on changes with the slider */
   /*
-  if (priv->timer_id > 0)
-    {
-      g_source_remove (priv->timer_id);
-    }
-  priv->timer_id = g_timeout_add (VOLUME_BEEP_DELAY, (GSourceFunc) volume_beep, vsp);
-  */
+     if (priv->timer_id > 0)
+     {
+     g_source_remove (priv->timer_id);
+     }
+     priv->timer_id = g_timeout_add (VOLUME_BEEP_DELAY, (GSourceFunc) volume_beep, vsp);
+   */
 }
 
 static void
@@ -953,10 +973,12 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   gtk_container_forall (GTK_CONTAINER (priv->brightness_slider), (GtkCallback) get_brightness_scale, vsp);
   gtk_range_set_increments (GTK_RANGE (priv->brightness_scale), 1, 1);
   gtk_range_set_range (GTK_RANGE (priv->brightness_scale), (priv->allow_zero && !priv->is_770) ? 0 : 1, priv->max_brightness);
-  gtk_range_set_update_policy (GTK_RANGE (priv->brightness_scale), GTK_UPDATE_DELAYED);
+  //gtk_range_set_update_policy (GTK_RANGE (priv->brightness_scale), GTK_UPDATE_DELAYED);
   hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->brightness_slider), priv->brightness_level);
   g_object_set (G_OBJECT (priv->brightness_slider), "has_mute", FALSE, NULL);
-  g_signal_connect (G_OBJECT (priv->brightness_slider), "level-changed", G_CALLBACK (brightness_level_changed), vsp);
+  g_signal_connect (G_OBJECT (priv->brightness_slider), "level-changed", G_CALLBACK (brightness_level_changing), vsp);
+  //gtk_widget_add_events(GTK_WIDGET(priv->brightness_scale), GDK_BUTTON_RELEASE_MASK);
+  //g_signal_connect (G_OBJECT (priv->brightness_scale), "button-release-event", G_CALLBACK (brightness_level_changed), vsp);
 
   priv->volume_slider = hildon_hvolumebar_new ();
   gtk_container_forall (GTK_CONTAINER (priv->volume_slider), (GtkCallback) get_volume_scale, vsp);
@@ -965,8 +987,8 @@ advanced_backlight_plugin_create_menu (AdvancedBacklightPlugin * vsp)
   hildon_volumebar_set_level (HILDON_VOLUMEBAR (priv->volume_slider), priv->volume_level);
   g_signal_connect (G_OBJECT (priv->volume_slider), "level-changed", G_CALLBACK (volume_level_changed), vsp);
   g_signal_connect (G_OBJECT (priv->volume_slider), "mute-toggled", G_CALLBACK (volume_mute_toggled), vsp);
-  gtk_widget_add_events(GTK_WIDGET(priv->volume_scale), GDK_BUTTON_RELEASE_MASK);
-  g_signal_connect(G_OBJECT(priv->volume_scale), "button-release-event", G_CALLBACK(volume_beep), vsp);
+  gtk_widget_add_events (GTK_WIDGET (priv->volume_scale), GDK_BUTTON_RELEASE_MASK);
+  g_signal_connect (G_OBJECT (priv->volume_scale), "button-release-event", G_CALLBACK (volume_beep), vsp);
 
 
   /* Create rotation buttons with rotated labels */
@@ -1347,10 +1369,10 @@ advanced_backlight_plugin_dsme_update_brightness_thread (gpointer d)
         {
           g_debug ("priv->brightness_level=%d\n", value);
 
-          if (!is_770 ())
-            advanced_backlight_plugin_dsme_update_brightness_value (priv, value);
-          else
+          if (priv->is_770)
             advanced_backlight_plugin_770_update_brightness_value (priv, value);
+          else
+            advanced_backlight_plugin_dsme_update_brightness_value (priv, value);
           prev_value = value;
         }
     }
@@ -1363,13 +1385,16 @@ advanced_backlight_plugin_770_update_brightness_value (AdvancedBacklightPluginPr
   FILE *myfile;
   value = value * 256 + 128;    /* Math stolen directly from Fanoush's blset script. was 256 + 128 */
 
+  g_debug ("%s\n", __PRETTY_FUNCTION__);
+
   myfile = fopen (SYSFS_BRIGHTNESS, "w");
 
   if (myfile != NULL)
-    {
-      fprintf (myfile, "%d\n", value);
-      fclose (myfile);
-    }
+  {
+    g_debug ("Setting 770 brightness to %d\n", value);
+    fprintf (myfile, "%d\n", value);
+    fclose (myfile);
+  }
 }
 
 /*
@@ -1588,7 +1613,7 @@ advanced_backlight_plugin_configure_applet (AdvancedBacklightPlugin * vsp)
       if (!priv->is_770)
         priv->allow_zero = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (allow_zero_checkbox));
       else
-	priv->allow_zero = 0;
+        priv->allow_zero = 0;
       if (!priv->is_n810 && priv->allow_zero)
         {
           confirmation_dialog =
